@@ -1,8 +1,8 @@
 /* vim: set sw=4: -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
-   rsvg.c: SAX-based renderer for SVG files into a GdkPixbuf.
+   rsvg-cond.c: Handle SVG conditionals
 
-   Copyright (C) 2004 Dom Lachowicz <cinamod@hotmail.com>
+   Copyright (C) 2004-2005 Dom Lachowicz <cinamod@hotmail.com>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -65,7 +65,7 @@ static const guint nb_implemented_extensions = 0;
 static int 
 rsvg_feature_compare(const void *a, const void *b)
 {
-	return strcmp((const char *)a, (const char *)b);
+	return strcmp((const char *)a, *(const char **)b);
 }
 
 /* http://www.w3.org/TR/SVG/struct.html#RequiredFeaturesAttribute */
@@ -134,13 +134,25 @@ rsvg_cond_parse_system_language (const char * value)
 		permitted = FALSE;
 
 #if defined(G_OS_WIN32)
-		locale = g_win32_getlocale ();
-#elif defined(HAVE_LC_MESSAGES)
-		locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-#else
-		/* catch-all */
-		locale = g_strdup (setlocale (LC_ALL, NULL));
+		if(!locale)
+			locale = g_win32_getlocale ();
+#endif       
+	
+		if(!locale)
+			locale = g_strdup (g_getenv ("LANG"));
+		
+#if defined(HAVE_LC_MESSAGES)
+		if(!locale)
+			locale = g_strdup (setlocale (LC_MESSAGES, NULL));
 #endif
+		
+		if(!locale)
+			locale = g_strdup (setlocale (LC_ALL, NULL));
+		
+		if(!locale || strcmp(locale, "C") == 0) {
+			g_free(locale);
+			locale = g_strdup("en");
+		}
 
 		if (locale)
 			{				
