@@ -68,9 +68,9 @@ typedef struct _RsvgIRect RsvgIRect;
 
 struct RsvgSaxHandler {
 	void (*free) (RsvgSaxHandler *self);
-	void (*start_element) (RsvgSaxHandler *self, const xmlChar *name, RsvgPropertyBag *atts);
-	void (*end_element) (RsvgSaxHandler *self, const xmlChar *name);
-	void (*characters) (RsvgSaxHandler *self, const xmlChar *ch, int len);
+	void (*start_element) (RsvgSaxHandler *self, const char *name, RsvgPropertyBag *atts);
+	void (*end_element) (RsvgSaxHandler *self, const char *name);
+	void (*characters) (RsvgSaxHandler *self, const char *ch, int len);
 };
 
 struct RsvgHandlePrivate {
@@ -221,7 +221,7 @@ typedef void (*RsvgPropertyBagEnumFunc) (const char * key,
 										 gpointer user_data);
 
 RsvgPropertyBag *
-rsvg_property_bag_new (const xmlChar **atts);
+rsvg_property_bag_new (const char **atts);
 
 void
 rsvg_property_bag_free (RsvgPropertyBag *bag);
@@ -342,6 +342,26 @@ void _rsvg_pop_view_box(RsvgDrawingCtx *ctx);
 
 void rsvg_SAX_handler_struct_init (void);
 
+void rsvg_return_if_fail_warning (const char *pretty_function,
+								  const char *expression,
+								  GError **error);
+
+#define rsvg_return_if_fail(expr, error)		G_STMT_START{			\
+     if G_LIKELY(expr) { } else       					\
+       {								\
+	 rsvg_return_if_fail_warning (__PRETTY_FUNCTION__,		        \
+		                   #expr, error);				\
+	 return;							\
+       };				}G_STMT_END
+
+#define rsvg_return_val_if_fail(expr,val,error)	G_STMT_START{			\
+     if G_LIKELY(expr) { } else						\
+       {								\
+	 rsvg_return_if_fail_warning (__PRETTY_FUNCTION__,		        \
+		                   #expr, error);				\
+	 return (val);							\
+       };				}G_STMT_END
+
 G_END_DECLS
 
 #endif
